"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import TestCase


class SimpleTest(TestCase):
    def test_basic_addition(self):
        """
        Tests that 1 + 1 always equals 2.
        """
        self.assertEqual(1 + 1, 2)




import requests
import json
from app_settings import *


def get_access_token(authorization_code):
    """
    Takes the temporary, short-lived authorization_code returned by tribe and
    sends it back (with some other ids and secrets) in exchange for an
    access_token.

    Arguments:
    authorization_code -- a string of characters returned by Tribe when it
    redirects the user from the page where they authorize the client to access
    their resources

    Returns:
    access_token -- another string of characters, with which users can remotely
    access their resources.

    """
    parameters = {"client_id": TRIBE_ID, "client_secret": TRIBE_SECRET, "grant_type": "authorization_code",  "code": authorization_code, "redirect_uri": TRIBE_REDIRECT_URI}
    tribe_connection = requests.post(ACCESS_TOKEN_URL, data=parameters)
    result = tribe_connection.json()
    if 'access_token' in result:
        access_token = result['access_token']
        return access_token
    else:
        return None

def retrieve_public_genesets(options={}):
    """
    Returns only public genesets
    """

    genesets_url = TRIBE_URL + '/api/v1/geneset/?format=json'

    for opt_key,opt in options.iteritems():
        genesets_url += '&'+opt_key+'='+opt

    try:
        tribe_connection = requests.get(genesets_url)
        result = tribe_connection.json()
        genesets = result['objects']
        return genesets

    except:
        return []


def retrieve_public_versions(options={}):
    """
    Returns only public versions
    """

    versions_url = TRIBE_URL + '/api/v1/version/?format=json'

    for opt_key,opt in options.iteritems():
        versions_url += '&'+opt_key+'='+opt

    try:
        tribe_connection = requests.get(versions_url)
        result = tribe_connection.json()
        versions = result['objects']
        return versions

    except:
        return []


def retrieve_user_object(access_token):
    """
    Makes a get request to tribe using the access_token to get the user's info
    (the user should only have permissions to see the user object that
    corresponds to them).

    Arguments:
    access_token -- The OAuth token with which the user has access to their
    resources. This is a string of characters.

    Returns:
    Either - 

    a) 'OAuth Token expired' if the access_token has expired,
    b) An empty list [] if the access_token is completely invalid, or
    c) The user object this user has access to (in the form of a dictionary)

    """

    parameters = {'oauth_consumer_key': access_token}

    try:
        tribe_connection = requests.get(TRIBE_URL + '/api/v1/user', params = parameters)
        result = tribe_connection.json()
        user = result['objects']  # This is in the form of a list
        meta = result['meta']

        if (meta.has_key('oauth_token_expired')):
            return ('OAuth Token expired')
        else:
            return user[0]  # Grab the first (and only) element in the list
    except:
        return []


def retrieve_user_genesets(access_token, options={}):
    """
    Returns genesets created by the user
    """

    try:
        get_user = retrieve_user_object(access_token)

        if (get_user == 'OAuth Token expired' or get_user == []):
            return ([])

        else:
            options['oauth_consumer_key'] = access_token
            options['creator'] = str(get_user['id'])
            options['show_tip'] = 'true'
            options['full_annotations'] = 'true'

            genesets_url = TRIBE_URL + '/api/v1/geneset/'

            tribe_connection = requests.get(genesets_url, params=options)
            result = tribe_connection.json()
            meta = result['meta']
            genesets = result['objects']
            return genesets

    except:
        return []


def retrieve_user_versions(access_token, geneset):
    """
    Returns all versions that belong to a specific geneset
    (if user has access to that geneset)
    """

    try:
        parameters = {'oauth_consumer_key': access_token}

        versions_url = TRIBE_URL + '/api/v1/version/?geneset__id=' + geneset + CROSSREF_DB
        tribe_connection = requests.get(versions_url, params=parameters)
        result = tribe_connection.json()
        meta = result['meta']
        versions = result['objects']
        return versions

    except:
        return []

def retrieve_all_user_versions(access_token):

    try:
        parameters = {'oauth_consumer_key': access_token}

        versions_url = TRIBE_URL + '/api/v1/version/?' + CROSSREF_DB + '&show_tip=true'
        tribe_connection = requests.get(versions_url, params=parameters)
        result = tribe_connection.json()
        meta = result['meta']
        versions = result['objects']
        return versions

    except:
        return []

def create_remote_geneset(access_token, geneset_info):

    if 'organism' in geneset_info:
        scientific_name = geneset_info['organism']
        organism_request = requests.get(TRIBE_URL + '/api/v1/organism?scientific_name=' + str(scientific_name))
        response = organism_request.json()
        organism = response['objects'][0]
        geneset_info['organism'] = organism['resource_uri']

    try:
        headers = {'AUTH': 'OAuth ' + access_token, 'Content-Type': 'application/json'}
        payload = json.dumps(geneset_info)
        genesets_url = TRIBE_URL + '/api/v1/geneset'
        r = requests.post(genesets_url, data=payload, headers=headers)
        response = r.json()
        return response

    except:
        return []

def create_remote_version(access_token, version_info):

    try:
        headers = {'AUTH': 'OAuth ' + access_token, 'Content-Type': 'application/json'}
        payload = json.dumps(version_info)
        versions_url = TRIBE_URL + '/api/v1/version'
        r = requests.post(versions_url, data=payload, headers=headers)
        response = r.json()
        return response
    except:
        return []

def return_user_object(access_token):
    parameters = {'oauth_consumer_key': access_token}
    tribe_connection = requests.get(TRIBE_URL + '/api/v1/user', params = parameters)

    try:
        result = tribe_connection.json()
        return result
    except:
        result = '{"meta": {"previous": null, "total_count": 0, "offset": 0, "limit": 20, "next": null}, "objects": []}'
        result = json.loads(result)
        return result


def obtain_token_using_credentials(username, password, client_id, client_secret):
    oauth_url = TRIBE_URL + '/oauth2/token/'
    payload = {'grant_type': 'password', 'username': username, 'password': password, 'client_id': client_id, 'client_secret': client_secret}
    r = requests.post(oauth_url, data=payload)
    tribe_response = r.json()
    return tribe_response['access_token']


from django.shortcuts import get_object_or_404, render, render_to_response, redirect
from django.http import HttpResponse
from django.template import Context, loader, RequestContext
from tribe_client import utils
from .app_settings import *
import json


def connect_to_tribe(request):
    if 'tribe_token' not in request.session:
        return render(request, 'establish_connection.html', {'tribe_url': TRIBE_URL, 'access_code_url': ACCESS_CODE_URL, 'client_id': TRIBE_ID, 'scope': 'write'})
    else:
        return display_genesets(request)

def logout_from_tribe(request):
    request.session.clear()
    return connect_to_tribe(request)

def get_token(request):
    access_code = request.GET.__getitem__('code')
    access_token = utils.get_access_token(access_code)
    request.session['tribe_token'] = access_token
    request.session['tribe_user'] = utils.retrieve_user_object(access_token)
    return redirect('display_genesets')

def display_genesets(request):
    if 'tribe_token' in request.session:
        access_token = request.session['tribe_token']
        get_user = utils.retrieve_user_object(access_token)

        if (get_user == 'OAuth Token expired' or get_user == []):
            request.session.clear()
            return connect_to_tribe(request)

        else:  # The user must be logged in and has access to her/himself
            genesets = utils.retrieve_user_genesets(access_token, {'full_genes': 'true', 'limit': 100})
            tribe_user = get_user
            return render(request, 'display_genesets.html', {'tribe_url': TRIBE_URL, 'genesets': genesets, 'tribe_user': tribe_user})

    else:
        return connect_to_tribe(request)

def display_versions(request, geneset):
    if 'tribe_token' in request.session:
        access_token = request.session['tribe_token']
        get_user = utils.retrieve_user_object(access_token)

        if (get_user == 'OAuth Token expired' or get_user == []):
            request.session.clear()
            return connect_to_tribe(request)

        else:
            versions = utils.retrieve_user_versions(access_token, geneset)
            for version in versions:
                version['gene_list'] = []
                for annotation in version['annotations']:
                    version['gene_list'].append(annotation['gene']['standard_name'])
            return render(request, 'display_versions.html', {'versions': versions})

def return_access_token(request):
    if 'tribe_token' in request.session:
        data = { 'access_token': request.session['tribe_token'] }
    else:
        data = { 'access_token': 'No access token' }
    data = json.dumps(data)
    return HttpResponse(data, content_type='application/json')

def create_geneset(request):

    geneset_info = request.POST.get('geneset')
    geneset_info = json.loads(geneset_info)
    genes = request.POST.get('genes')
    genes = genes.split(",")
    num_genes = len(genes)
    geneset_info['selectedGenes'] = genes
    geneset_info['xrdb'] = CROSSREF
    geneset_info['description'] = 'Initial version containing the first ' + str(num_genes) + ' genes.'

    if 'tribe_token' in request.session:
        tribe_token = request.session['tribe_token']
        is_token_valid = utils.retrieve_user_object(tribe_token)
        if (is_token_valid =='OAuth Token expired'):
            tribe_token = None
            tribe_response = {'response': 'Not Authorized'}
        else:
            tribe_response = utils.create_remote_geneset(tribe_token, geneset_info)
            slug = tribe_response['slug']
            creator = tribe_response['creator']['username']

            geneset_url = TRIBE_URL + "/#/use/detail/" + creator + "/" + slug
            tribe_response = {'geneset_url': geneset_url}


    else:
        tribe_token = None
        tribe_response = {'response': 'Not Authorized'}

    json_response = json.dumps(tribe_response)

    return HttpResponse(json_response, content_type='application/json')

def return_user_obj(request):

    if 'tribe_token' in request.session:
        tribe_token = request.session['tribe_token']

    else:
        tribe_token = None

    tribe_response = utils.return_user_object(tribe_token)

    json_response = json.dumps(tribe_response)
    return HttpResponse(json_response, content_type='application/json')


from django.conf.urls import *
from tribe_client import views

urlpatterns = patterns('',
    url(r'^$', views.connect_to_tribe, name='connect_to_tribe'),
    url(r'^logout$', views.logout_from_tribe, name='logout_from_tribe'),
    url(r'^get_token$', views.get_token, name='get_token'),
    url(r'^display_genesets/$', views.display_genesets, name='display_genesets'),
    url(r'^display_geneset_versions/(?P<geneset>[-_\w]+)/$', views.display_versions, name='display_versions'),
    url(r'^token$', views.return_access_token, name='return_access_token'),
    url(r'^return_user$', views.return_user_obj, name='return_user'),
    url(r'^create_geneset$', views.create_geneset, name='create_geneset'),

)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
#from distutils.core import setup
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name = 'django-tribe-client',
    version = '1.1.1',
    author = 'Rene A. Zelaya',
    author_email = 'tribe.greenelab@gmail.com',
    packages = ['tribe_client'],
    include_package_data = True,
    url = 'https://bitbucket.org/greenelab/tribe-client',
    license = 'LICENSE.txt',
    description = 'Reusable Django app to connect servers with the Tribe web service at the University of Pennsylvania',
    long_description = open('README.rst').read(),
    install_requires = [
        'requests == 2.5.0',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
    ],
)



from django.conf import settings

TRIBE_URL = getattr(settings, 'TRIBE_URL', 'http://tribe.greenelab.com')

ACCESS_CODE_URL = TRIBE_URL + "/oauth2/authorize/"
ACCESS_TOKEN_URL = TRIBE_URL + "/oauth2/token/"

ENTREZ = 'Entrez'

CROSSREF_DB = '&xrdb=' + getattr(settings, 'TRIBE_CROSSREF_DB', ENTREZ)
CROSSREF = getattr(settings, 'TRIBE_CROSSREF_DB', ENTREZ)

TRIBE_ID = getattr(settings, 'TRIBE_ID', '')
TRIBE_SECRET= getattr(settings, 'TRIBE_SECRET', '')

TRIBE_REDIRECT_URI = getattr(settings, 'TRIBE_REDIRECT_URI', '')
